import { defaults } from './config';

class JqNavScrollSpy {
  constructor(element, configs) {
    this.$element = element;
    this.$win = $(window);
    this.defaults = $.extend({}, defaults, configs);
    this.init();
  }

  // 初始化
  init() {
    this.$navItems = $(this.defaults.navItems);
    this.$spyItems = $(this.defaults.spyItems);
    this.$scrollContainer = $(this.defaults.scrollContainer);
    // 修正初始化的时候元素距离顶部的距离不等于滚动的距离的变量，也就是减去第一个元素距离顶部的高度
    this.fixTop = $(this.$spyItems[0]).offset().top;
    this.spyItemsData = this.getSpyItemsData();
    this.spyScroll();
    this.clickSwitch();
  }

  // 监听滚动事件
  spyScroll() {
    this.$win.on('scroll', this.throttle(this.scrollCallBack, 100, 200));
  }

  // 滚动监听的回调函数
  scrollCallBack() {
    const spyIndex = this.getVisibleElIndex();
    this.changeNav(this.$navItems[spyIndex]);
  }

  // 存储监视滚动元素的中心位置数组
  getSpyItemsData() {
    const spyItemsData = [];
    this.$spyItems.each((index) => {
      spyItemsData[index] = this.getOffsetTop(index) + $(this.$spyItems[index]).height() / 2;
    });
    return spyItemsData;
  }

  // 获得当前滚动到视图区的元素的索引
  getVisibleElIndex() {
    let spyIndex;
    const scrollTop = parseInt(this.$win.scrollTop(), 10);
    $.each(this.spyItemsData, (index) => {
      if (this.spyItemsData[0] >= scrollTop) {
        spyIndex = 0;
        return false;
      } if (this.spyItemsData[index] <= scrollTop && scrollTop <= this.spyItemsData[index + 1]) {
        spyIndex = index + 1;
        return false;
      }
      return true;
    });
    return spyIndex;
  }

  // 节流函数
  throttle(func, wait, mustRun) {
    let timeout;
    let startTime = new Date();
    return (...rest) => {
      const curTime = new Date();
      clearTimeout(timeout);
      // 如果达到了规定的触发时间间隔，触发 handler
      if (curTime - startTime >= mustRun) {
        func.apply(this, rest);
        startTime = curTime;
        // 没达到触发间隔，重新设定定时器
      } else {
        timeout = setTimeout(func.bind(this), wait);
      }
    };
  }

  // 点击切换
  clickSwitch() {
    const self = this;
    this.$navItems.on('click', function onNavItemClick() {
      self.changeNav(this);
      const navIndex = $(this).index();
      self.$win.off('scroll');
      self.scrollIntoView(navIndex);
    });
  }

  // 改变导航active
  changeNav(currentNav) {
    this.$navItems.removeClass('active');
    $(currentNav).addClass('active');
  }

  // 滚动到可视区
  scrollIntoView(navIndex) {
    const offsetTop = parseInt(this.getOffsetTop(navIndex), 10);
    if (!this.$scrollContainer.is(':animated')) {
      this.$scrollContainer.stop()
        .animate({ scrollTop: offsetTop }, this.defaults.speed,
          this.defaults.easing, () => {
            // 动画结束后重新注册滚动监听事件
            this.spyScroll();
          });
    }
  }

  // 获取滚动元素距离顶部的距离
  getOffsetTop(index) {
    return parseInt($(this.$spyItems[index]).offset().top, 10) - parseInt(this.fixTop, 10);
  }
}
export default JqNavScrollSpy;
